@@ -1,15 +1,15 @@
-import asyncio
 import json
 import os
 import re
 import sys
 import threading
 import typing
-from dataclasses import asdict, dataclass, is_dataclass
+from dataclasses import asdict, is_dataclass
 from types import SimpleNamespace
 from typing import Any, Callable, Dict, List, Optional, Union
 
 import socketio
+from pydantic.dataclasses import dataclass
 
 from .bot_types import *
 
@@ -19,6 +19,8 @@ from .bot_types import *
 # else:
 #     print("No arguments provided.")
 
+current_args = {}
+
 
 @dataclass
 class Call:
@@ -46,15 +48,11 @@ bot_id = json_data["botId"] if json_data is not None else None
 conversation_id = json_data["conversationId"] if json_data is not None else None
 """Conversation ID"""
 
-thread_id = json_data["conversationThreadId"] if json_data is not None else None
-"""Conversation Thread ID"""
-
 bot_context = (
     {
         "botId": json_data["botId"],
         "botCodeId": json_data["botCodeId"],
         "conversationId": json_data["conversationId"],
-        "conversationThreadId": json_data["conversationThreadId"],
         "chargeUserIds": json_data["chargeUserIds"],
     }
     if json_data is not None
@@ -113,19 +111,12 @@ def live_user_visible_arg_map(dict: Dict[Any, Any]):
     }
 
 
-def thread_arg_map(dict: Dict[Any, Any]):
-    return {
-        "thread": Thread(**dict["thread"]),
-    }
-
-
 arg_map: Dict[str, Callable[[Dict[Any, Any]], Dict[Any, Any]]] = {
     "messageDirect": message_arg_map,
     "messageAdd": message_arg_map,
     "conversationStart": conversation_arg_map,
     "conversationUserAdd": conversation_arg_map,
     "userVisible": live_user_visible_arg_map,
-    "threadStop": thread_arg_map,
 }
 
 
@@ -144,7 +135,7 @@ def convert_keys_to_snake_case(data):
 
 def convert_to_dict(data):
     if is_dataclass(data):
-        return convert_to_dict(asdict(data))
+        return convert_to_dict(asdict(data))  # type: ignore
     elif isinstance(data, SimpleNamespace):
         return convert_to_dict(data.__dict__)
     elif isinstance(data, dict):
@@ -199,6 +190,8 @@ def start():
 
 
 def run_call(message: str):
+    global current_args
+
     msg = typing.cast(Any, convert_keys_to_snake_case(json.loads(message)))
     log("[BOT] message", msg)
     funcName = msg.get("func")
@@ -207,9 +200,10 @@ def run_call(message: str):
     if func is not None:
         arg_mapper = arg_map.get(funcName)
         if arg_mapper is not None:
-            func(**arg_mapper(funcParams))
+            current_args = arg_mapper(funcParams)
         else:
-            func(**funcParams)
+            current_args = funcParams
+        func(**current_args)
 
 
 def start_nonblocking():
@@ -383,7 +377,11 @@ def message_typing() -> None:
     """
     call_no_return(
         "botCodeMessageTyping",
-        {},
+        {
+            "parent_message_id": getattr(
+                current_args.get("message", None), "parent_message_id", None
+            ),
+        },
     )
 
 
@@ -402,11 +400,13 @@ def message_send(
     mood: Optional[Mood] = None,
     impersonate_user_id: Optional[str] = None,
     files: Optional[List[File]] = None,
-    thread: Optional[Thread] = None,
 ) -> Message:
     """
     Send a message to the active conversation
     """
+
+    print(current_args)
+
     return Message(
         **call_return(
             "botCodeMessageSend",
@@ -425,7 +425,9 @@ def message_send(
                 "mood": mood,
                 "impersonate_user_id": impersonate_user_id,
                 "file_ids": [file.id for file in files] if files is not None else None,
-                "thread": thread,
+                "parent_message_id": getattr(
+                    current_args.get("message", None), "parent_message_id", None
+                ),
             },
         )
     )
@@ -529,7 +531,6 @@ def message_history(
     limit: Optional[int] = None,
     start: Optional[int] = None,
     include_hidden: Optional[bool] = None,
-    thread_id: Optional[str] = None,
 ) -> List[Message]:
     """
     Get messages from the active conversation
@@ -541,7 +542,6 @@ def message_history(
             "limit": limit,
             "start": start,
             "include_hidden": include_hidden,
-            "thread_id": thread_id,
         },
     )
 
@@ -796,21 +796,6 @@ def conversation_buttons_show(
     )
 
 
-def tool_context_menu_set(
-    user_id: str, menu_items: Optional[List[MenuItem]] = None
-) -> None:
-    """
-    Add context menu to the current web page all of the time
-    """
-    call_no_return(
-        "botCodeToolContextMenuSet",
-        {
-            "user_id": user_id,
-            "menu_items": menu_items,
-        },
-    )
-
-
 def tool_conversation_show(
     session_id: str,
     video_call_enabled: Optional[bool] = None,
@@ -850,7 +835,6 @@ def file_create(
     thumbnail: Optional[Image] = None,
     lang: Optional[UserLang] = None,
     indexable: Optional[bool] = None,
-    characters: Optional[Dict[str, Character]] = None,
 ) -> File:
     """
     Create file
@@ -866,7 +850,6 @@ def file_create(
                 "thumbnail": thumbnail,
                 "lang": lang,
                 "indexable": indexable,
-                "characters": characters,
             },
         )
     )
@@ -892,90 +875,6 @@ def file_update(
     )
 
 
-def file_book_character_set(
-    file_id: str,
-    character_id: str,
-    image: Image,
-    name: str,
-    voice_id: str,
-) -> None:
-    """
-    Set book character
-    """
-    call_no_return(
-        "botCodeFileBookCharacterSet",
-        {
-            "file_id": file_id,
-            "character_id": character_id,
-            "image": image,
-            "name": name,
-            "voice_id": voice_id,
-        },
-    )
-
-
-def file_book_page_add(
-    file_id: str,
-    image: Optional[Image] = None,
-) -> int:
-    """
-    Create book page
-    """
-    result: Dict[Any, Any] = call_return(
-        "botCodeFileBookPageSet",
-        {
-            "file_id": file_id,
-            "image": image,
-        },
-    )
-
-    return result.get("page_index", 0)
-
-
-def file_book_page_line_add(
-    file_id: str,
-    page_index: int,
-    character_id: str,
-    markdown: str,
-) -> int:
-    """
-    Create book page line
-    """
-    result: Dict[Any, Any] = call_return(
-        "botCodeFileBookPageLineSet",
-        {
-            "file_id": file_id,
-            "page_index": page_index,
-            "line": {
-                "character_id": character_id,
-                "markdown": markdown,
-            },
-        },
-    )
-
-    return result.get("line_index", 0)
-
-
-def file_book_user_context_set(
-    file_id: str,
-    page_index: Optional[int] = None,
-    line_index: Optional[int] = None,
-) -> int:
-    """
-    Create book page line
-    """
-    return call_return(
-        "botCodeFileBookUserContextSet",
-        {
-            "file_id": file_id,
-            "fields": {
-                "page_index": page_index,
-                "line_index": line_index,
-            },
-        },
-    ).line_index
-
-
 def file_to_text_gen_message(
     file: File,
     role: Optional[TextGenRole] = None,
